% This script takes the solutions matrix generated in
% produceHistories.m, and models the 14C and 10Be concentrations that 
% result from each history. It then saves the exposure histories
% that produce nuclide concentrations within a specified uncertainty of
% measured Rwenzori data. The 14C and 10Be concentrations are modeled in a 5m depth
% profile and the simulations are run at a range of user-specified erosion 
% rates.

% The input parameters on lines 12-24 need to be specified to run the code.

% Enter your data
load('prod_rates_LSD_RZ.mat') % production rate profiles
load('exposureBurialHistories.mat') % exposure histores from produceHistories.m

sampleNames = ["RZ01", "RZ02", "RZ03"];
sampleTarget10Be = [16.57*10^4, 16.89*10^4, 18.89*10^4]; % atoms/g
sampleuncertaintyBe = [0.18*10^4, 0.18*10^4, 0.17*10^4]; % atoms/g
sampleTarget14C = [13.81*10^4, 18.09*10^4, 18.13*10^4]; % atoms/g
sampleuncertainty14C = [0.77*10^4, 1.01*10^4, 1,02*10^4]; % atoms/g
sampleDensity = [2.67, 2.67, 2.67]; % g/cm3
sampleThickness = [19, 14, 28]; % mm
sampleShielding = [0.969, 0.97, 0.97];
erosionRate = 0:10:200; % mm/century
fileName = "monteCarloResults_RZ.mat"; % output file that will contain solutions

% Combine all input data into a structure that can be iterated through
samples.names = sampleNames; % Names of samples
samples.target10Be = sampleTarget10Be; % 10Be concentrations
samples.uncertainty10Be = sampleuncertaintyBe; % 10Be analytical uncertainty
samples.target14C = sampleTarget14C; % 14C concentrations
samples.uncertaintyC = sampleuncertainty14C; % 14C analytical uncertainty
samples.density = sampleDensity; % Sample densities
samples.thickness = sampleThickness; % Sample thicknesses
samples.shielding = sampleShielding; % Sample shielding factors

% Run monte carlo simulations 
for n = 1:length(samples.names) % for each sample    
    
    % Define acceptable nuclide concentrations and production rates
    target10Be = samples.target10Be(n); % measured 10Be
    uncertainty10Be = samples.uncertainty10Be(n); % 10Be analytical uncertainty
    target14C = samples.target14C(n); % measured 14C
    uncertainty14C = samples.uncertaintyC(n); % 14C analytical uncertainty
    uncertainty10Be = sqrt((uncertainty10Be^2)+((.083*target10Be)^2)); % combined analytical and production rate (Borchers et al., 2016) uncertainty
    uncertainty14C = sqrt((uncertainty14C^2)+((.073*target14C)^2)); % combined analytical and production rate (Borchers et al., 2016) uncertainty
    max10Be = target10Be + (3 * uncertainty10Be); % maximum acceptable 10Be concentration (3 sigma)
    min10Be = target10Be - (3 * uncertainty10Be); % minimum acceptable 10Be concentration (3 sigma) 
    max14C = target14C + (3 * uncertainty14C); % maximum acceptable 14C concentration (3 sigma) 
    min14C = target14C - (3 * uncertainty14C); % minimum acceptable 14C concentration (3 sigma)
    density = samples.density(n); % sample density (g/cm3)
    depth = eval(samples.names(n) + ".z") / density; % converts mass depth to actual depth (cm)
    sampleThickness = samples.thickness(n); % sample thickness (mm)
    shielding = samples.shielding(n); % shielding factor 
    productionRate10 = eval(samples.names(n) + ".P10_tot") .* shielding; % 10Be production rate corrected for shielding
    productionRate14 = eval(samples.names(n) + ".P14_tot") .* shielding; % 14C production rate corrected for shielding
    beDecay = 4.99*(10^-7); % 10Be decay constant (per year)
    cDecay = 1.21*(10^-4); % 14C decay constant (per year)
    timeStep = 0:100:((length(exposureBurialHistories(1,:))-1)*100); % produce a time vector with century time steps
    depth2 = 0:1:5000; % depths from 0 to 5000 mm
    productionRate10 = interp1(depth*10, productionRate10, depth2); % interpolates 10Be production rate profile from cm to mm depth steps
    productionRate14 = interp1(depth*10,productionRate14,depth2); % Interpolate 14C production rate profile from cm to mm depth steps
    
    % Create empty matrices to populate with results
    final10Be = nan(1, 11000000); % empty matrix for saving 10Be concentrations produced by input exposure histories
    final14C = nan(1, 11000000); % empty matrix for saving 14C concentrations produced by input exposure histories
    solutionMatrix = NaN(11000000, 117); % empty matrix for saving exposure histories that produced viable solutions
    erosionRateSolutions = NaN(1, 11000000); % empty matrix for saving erosion rates that correspond to each viable solution
    
    % Calculate nuclide concentrations through time
    counter = 1; % initialize counter of how many solutions have been identified and is used to place solutions in correct positions in outputs
    for j = 1:length(exposureBurialHistories) % for each exposure history
        burialVector = exposureBurialHistories(j,:); % extract the exposure history
        for k = erosionRate % model nuclide concentrations for each erosion rate 
            concentration10Be = zeros(1,length(depth2)); % initialize depth profile to 0 nuclides for 10Be
            concentration14C = zeros(1,length(depth2)); % initialize depth profile to 0 nuclides for 14C
            for i = timeStep % for each time step
                concentration10Be = concentration10Be * exp(-beDecay * 100) + (productionRate10 * 100) * burialVector(i / 100 + 1); % concentration from last step * decay + production (if exposed)
                concentration14C = concentration14C * exp(-cDecay * 100) + (productionRate14 * 100) * burialVector( i / 100 + 1); % concentration from last step * decay + production (if exposed)      
                if burialVector(i / 100 + 1) == 0 && k > 0 % if the sample is currently buried and the erosion rate is greater than zero
                    concentration10Be = [concentration10Be(1 + k:end), zeros(1, k)]; % Remove the top k number of mm from the 10Be depth profile and add k zeros to the bottom of the depth profile
                    concentration14C = [concentration14C(1 + k:end), zeros(1, k)]; % Remove the top k number of mm from the 14C depth profile and add k zeros to the bottom of the depth profile
                end
            end

            % Determine if exposure history is consistent with measured nuclide concentrations
            concentration14Test = mean(concentration14C(1:sampleThickness)); % calculate simulated 14C concentration over sample thickness
            concentration10Test = mean(concentration10Be(1:sampleThickness)); % calculate simulated 10Be concentration over sample thickness
            if concentration14Test >= min14C && concentration14Test <= max14C && concentration10Test <= max10Be && concentration10Test >= min10Be % if simulated concentrations are within uncertainty of measured concentrations
                solutionMatrix(counter,:) = burialVector(1,:); % save the exposure history that provided a viable solution
                final14C(counter) = concentration14Test; % save the corresponding final 14C concentration
                final10Be(counter)  =  concentration10Test; % save the corresponding final 10Be concentration
                erosionRateSolutions(counter) = k; % save the corresponding erosion rate
                counter = counter + 1; % advance counter by 1 so next solution is saved in appropriate position
            end
        end
    end
    
    % Clean up the solution matrices
    % Variables were created to be much larger than likely necessary. Here,
    % we remove the unfilled data storage (NaNs) from these variables.
    erosionRateSolutions(isnan(erosionRateSolutions)) = []; % Remove all NaN values from erosionRateSolutions
    final14C(isnan(final14C)) = []; % Remove all NaN values from final14C
    final10Be(isnan(final10Be)) = []; % Remove all NaN values from final10Be
    % Remove all extra rows in solutionMatrix
    check = isnan(solutionMatrix);
    check1 = find(check, 1);
    solutionMatrix = solutionMatrix(1:check1 - 1,:);
    % Create or edit a structure of the solution data
    monteCarloResults.(samples.names(n)).exposureBurialSolutions = solutionMatrix;
    monteCarloResults.(samples.names(n)).erosionRateSolutions = erosionRateSolutions;
    monteCarloResults.(samples.names(n)).final14C = final14C;
    monteCarloResults.(samples.names(n)).final10Be = final10Be;
end

% Save solution data
save(fileName, 'monteCarloResults', '-v7.3')