% This script shows how we made figures with our Quelccaya data. Additional
% visual processing was done in Adobe Illustrator. The user should adjust
% the inputs in lines 6-9.

% Load the data and graph titles
load('monteCarloResults_Q.mat') % previously-produced output from monteCarlo.m
samplenames4graph = ["Q-2-03", "Q-3-03", "Q-4-03", "Q-80"]; % sample names
exposureBurialGraphName = "Possible Quelccaya exposure histories"; % title for exposure plot
erosionPlotGraphName = "Quelccaya erosion rates"; % title for erosion plot

tempstruct = monteCarloResults; % make copy of data for manipulation
names = fieldnames(monteCarloResults); % extract the names of each sample 

% Find exposure histories common to each sample. This loop iterates through pairs of samples one at a time
% until it has reached the last sample. Because we're looking for histories
% common to all samples, we simply compare histories from each sample to
% the first sample in the list, one at a time.
for i = 1:length(fieldnames(monteCarloResults)) - 1 % for each pair of samples
    if i == 1 % on the first iteration
        solutionA = monteCarloResults.(names{i}).exposureBurialSolutions; % assign histories to a variable
    end
    solutionB = monteCarloResults.(names{i+1}).exposureBurialSolutions; % assign histories from the subsequent sample to a variable
    answers = intersect(solutionA, solutionB, 'rows'); % find common rows from the histories
    answerspt2 = ismember(solutionA, answers, 'rows'); % find the indices of the common histories
    solutionA = solutionA(answerspt2, :); % use indices to extract and save the histories so far
end
histories = unique(solutionA, 'rows'); % only report unique histories
tempstruct.histories4plot.exposureBurialSolutions = histories; % save this matrix in the duplicate structure for future iterating
names = fieldnames(tempstruct); % create array with the names of each sample analyzed in the loaded data that can be iterated over
samplenames4graph = [samplenames4graph, "Overlap"]; % this is added so the "overlap" column can be properly named in the figure

% Produce exposure history plot for each sample
for i = 1:length(names) % for each sample 
    tempstruct.(names{i}) = unique(tempstruct.(names{i}).exposureBurialSolutions, 'rows'); % find the unique histories and assign to a variable
    exposureDuration =  sum(tempstruct.(names{i}), 2); % add up all of the 1s in each row, which calculates the centuries of exposure in each history
    tempstruct.(names{i}) = [exposureDuration, tempstruct.(names{i})]; % adds this exposure total as an integer inserted in the first entry in each solution row
    tempstruct.(names{i}) = sortrows(tempstruct.(names{i})); % sort rows by exposure duration
end

figure
for j = 1:length(names)
    sgtitle(exposureBurialGraphName)
    subplot(length(names)+1,1,j)
    % When plotting, ignore the "exposure duration" integer added to front
    % of each scenario  by omitting the first value in each row. imagesc
    % produces a visualization of the rows where 1 is yellow and 0 is blue.
    % Assigning the image to "im" allows one to adjust the saturation on a
    % scale between 0-1.
    im = imagesc(tempstruct.(names{j})(:, 2:end));
    im.AlphaData = .75;
    set(gca, 'box', 'off')
    title(names{j})
    set(gca, 'YTick', [])
    set(gca, 'Xtick', [])
    title(samplenames4graph{j})
end

% Produce an exposure probability plot for each time interval by first calculating
% the mean in each column of the solutions that were found in each sample.
% This calculates the fraction of solutions that showed exposure for each
% century.
histories4plot = mean(tempstruct.histories4plot, 1);
% Repeat previous plotting code for this new vector
subplot(length(names)+1,1,j+1)
imagesc(histories4plot(:, 2:end))
set(gca,'box','off')
set(gca,'YTick', [])
title('Probability of exposure')
xticks(0:10:120) % specifies the location along the x axis where ticks are placed
xticklabels({'11','10','9','8','7','6','5','4','3','2','1'}) % specifies the labels to be placed in the locations specified in the line above
xlabel('Time (ka)') 

% Make a histogram of erosion rates for each sample
figure
hold on
for k = 1:length(names)-1
    % Erosion rates are multiplied by 10 to convert to units of cm/century
    histogram(monteCarloResults.(names{k}).erosionRateSolutions.*10,...
        'Normalization', 'probability', 'DisplayStyle', 'stairs', 'BinWidth', 10)
end

title(erosionPlotGraphName)
xlim([0 2500]) % User may want to adjust this value to their maximum erosion rate
xlabel('Erosion rate (cm/century)')
ylabel('Probability')
legend(samplenames4graph(1:end-1));