% This script generates a series of unique glacier exposure 
% histories, which are used as input to the monteCarlo.m script. Each time 
% step in a history is assigned exposure (1) or burial (0) based on a specified
% probability (P) that it matches the designation (burial or exposure) of 
% the previous time step. We used a range of P values to generate histories
% with varying amounts of autocorrelation. 

% The input parameters on lines 11-15 need to be specified to run the code.

% Define input parameters
nHistories = 100000; % number of histories to generate
nCenturies = 110; % number of centuries in each history
minP = 0.61; % minimum P value
maxP = 0.99; % maximum P value
nHistoriesperP = 4000; % number of histories per P value

% Calculate range of P values
interval = (maxP - minP) / (nHistories / nHistoriesperP - 1);
pValues = minP : interval : maxP;

% Generate an empty matrix to populate with calculated histories
exposureBurialHistories = NaN(nHistories,nCenturies);

% Initialize iterative counters
i = 1; % counts through all histories
j = 1; % counts through histories per P value
k = 1; % counts through P values

% Iterate until all histories are produced
while i <= nHistories
    
    % Generate a single history
    exposureBurialHistory = NaN(1, length(nCenturies)); % empty history to fill
    exposureBurialHistory(1) = randi(2, 1) - 1; % randomly fill 0 or 1 in first time step
    counter = 1; % counts through all time steps in history
    while length(exposureBurialHistory) ~= nCenturies % iterate until all time steps are filled
        counter = counter + 1; % advance to next time step
        if rand < pValues(k) % determine if value (0 or 1) is the same as in the previous time step based on P value
            exposureBurialHistory(counter) = exposureBurialHistory(counter - 1); % same as previous value
        else            
            exposureBurialHistory(counter) = abs(exposureBurialHistory(counter-1) - 1); % opposite of previous value
        end
    end
    
    % Determine if history is unique and save it if so
    temp = abs(exposureBurialHistories - repmat(exposureBurialHistory, [nHistories, 1])); % compares new history to all previous histories
    if min(sum(temp, 2)) ~= 0 % if new history is unique
        exposureBurialHistories(i, :) = exposureBurialHistory; % save the new history and index the counters
        i = i + 1;
        j = j + 1;
    end
    
    % Once the proper number of histories have been generated for a given
    % P value, advance k to move to next P value. 
    if j / (((nHistories / (length(pValues))))+1) == 1
        k = k + 1;
        j = 1;       
    end
end

% Save matrix for use in monteCarlo.m
save('exposureBurialHistories.mat', 'exposureBurialHistories')